unit MsBackGround;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, Controls,
  MclGraphics;

type

  TMsBackGround = class(TComponent)
  private
    FBitmap: TBitmap;
    FPosImage: TPositionImage;
    FActiveGradient: Boolean;
    FGradientDirection: TGradientDirection;
    FColor: TColor;
    FGradientColor: TColor;
    FActive: Boolean;
    FScrollDisable: Boolean;
    procedure InternalClientProc(var Msg: TMessage);
    procedure SetBitmap(Value: TBitmap);
    procedure SetPosImage(const Value: TPositionImage);
    procedure SetActiveGradient(const Value: Boolean);
    procedure SetColor(const Value: TColor);
    procedure SetGradientColor(const Value: TColor);
    procedure SetGradientDirection(const Value: TGradientDirection);
    procedure SetActive(const Value: Boolean);
  protected
    FForm: TForm;
    FWndDraw: THandle;
    FClientInstance: TFarProc;
    FPrevClientProc: TFarProc;
    procedure Loaded; override;
    procedure Invalidate;
    function CanDraw: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Active: Boolean read FActive write SetActive default True;
    property ActiveGradient: Boolean read FActiveGradient write SetActiveGradient default False;
    property Bitmap: TBitmap read FBitmap write SetBitmap;
    property Color: TColor read FColor write SetColor default clNone;
    property GradientColor: TColor read FGradientColor write SetGradientColor default clNone;
    Property GradientDirection: TGradientDirection read FGradientDirection write SetGradientDirection default gdHorizontal;
    property PosImage: TPositionImage read FPosImage write SetPosImage default piCenter;
    property ScrollDisable: Boolean read FScrollDisable write FScrollDisable default True;
  end;

implementation

// -----------------------------------------------------------------------------
// TMsBackGround ---------------------------------------------------------------
// -----------------------------------------------------------------------------

function TMsBackGround.CanDraw: Boolean;
begin
  Result := Active and (0 < FWndDraw) and (ActiveGradient or not FBitmap.Empty);
end;

constructor TMsBackGround.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FPosImage := piCenter;
  FBitmap := TBitmap.Create;

  FActive := True;
  FActiveGradient := False;
  FColor := clNone;
  FGradientColor := clNone;
  FGradientDirection := gdHorizontal;
  FScrollDisable := True;

  FWndDraw := 0;
  if AOwner is TForm then
    FForm := TForm(AOwner)
  else
    FForm := nil;
end;

destructor TMsBackGround.Destroy;
begin
  SetWindowLong(FWndDraw, GWL_WNDPROC, Integer(FPrevClientProc));
  Classes.FreeObjectInstance(FClientInstance);   
  FBitmap.Free;
  inherited Destroy;
end;
 
procedure TMsBackGround.InternalClientProc(var Msg: TMessage);
const
  Mode: array[TGradientDirection] of Cardinal
    = (GRADIENT_FILL_RECT_H, GRADIENT_FILL_RECT_V);
var
  DstDC: hDC;
  WorkArea: TRect;
begin
  case Msg.Msg of
    WM_NCCALCSIZE: { Desactivar Scroll }
      if ScrollDisable and ((GetWindowLong(FWndDraw, GWL_STYLE) and
        (WS_HSCROLL or WS_VSCROLL)) <> 0) Then
        SetWindowLong(FWndDraw, GWL_STYLE, GetWindowLong(FWndDraw, GWL_STYLE)
          and not (WS_HSCROLL or WS_VSCROLL));
{    WM_NCHITTEST:
      if CanDraw and (Msg.Result = HTCLIENT) then
        Msg.Result := HTTRANSPARENT;}
    WM_HSCROLL, WM_VSCROLL, WM_SIZE:
      if CanDraw then
        Invalidate;{InvalidateRect(FWndDraw, nil, False);{}
    WM_ERASEBKGND:
      if CanDraw then
      begin
        GetClientRect(FWndDraw, WorkArea);
        DstDc := TWMEraseBkGnd(Msg).DC;
        if (ActiveGradient and DrawGradientRect(DstDC, Color, GradientColor,
            WorkArea, GradientDirection))
          or (not ActiveGradient
            and DrawBitmap(DstDC, Bitmap, WorkArea, FForm.Color, PosImage)) then
        begin
          Msg.Result := 1;
          Exit;
        end;
      end;
  end;

  Msg.Result := CallWindowProc(FPrevClientProc, FWndDraw,
    Msg.Msg, Msg.wParam, Msg.lParam);
end;

procedure TMsBackGround.Invalidate;
begin
  if not Assigned(FForm) then
    Exit;

  if FForm.ClientHandle = 0 then
    FForm.Invalidate
  else
    InvalidateRect(FWndDraw, nil, True);
end;

procedure TMsBackGround.Loaded;
begin
  inherited Loaded;

  if (csDesigning in ComponentState) or (FForm = nil) then
    Exit;

  if FForm.ClientHandle = 0 then
    FWndDraw := FForm.Handle
  else
    FWndDraw := FForm.ClientHandle;

  FClientInstance := Classes.MakeObjectInstance(InternalClientProc);
  FPrevClientProc := Pointer(SetWindowLong(FWndDraw, GWL_WNDPROC,
    Integer(FClientInstance)));
end;

procedure TMsBackGround.SetActive(const Value: Boolean);
begin
  if Active <> Value then
  begin
    FActive := Value;
    Invalidate;
  end;
end;

procedure TMsBackGround.SetActiveGradient(const Value: Boolean);
begin
  if ActiveGradient <> Value then
  begin
    FActiveGradient := Value;
    Invalidate;
  end;
end;

procedure TMsBackGround.SetBitmap(Value: TBitmap);
begin
  FBitmap.Assign(Value);
  Invalidate;
end;

procedure TMsBackGround.SetColor(const Value: TColor);
begin
  if Color <> Value then
  begin
    FColor := Value;
    Invalidate;
  end;
end;

procedure TMsBackGround.SetGradientColor(const Value: TColor);
begin
  if GradientColor <> Value then
  begin
    FGradientColor := Value;
    Invalidate;
  end;
end;

procedure TMsBackGround.SetGradientDirection(const Value: TGradientDirection);
begin
  if GradientDirection <> Value then
  begin
    FGradientDirection := Value;
    Invalidate;
  end;
end;

procedure TMsBackGround.SetPosImage(const Value: TPositionImage);
begin
  if FPosImage <> Value then
  begin
    FPosImage := Value;
    Invalidate;
  end;
end;

end.
